@@ -19,18 +19,16 @@
  * USA
  *
  * --------------------------------------------------------------------------
- * $Id: JManagedConnection.java 20401 2010-10-04 11:58:23Z alitokmen $
+ * $Id: JManagedConnection.java 21462 2011-06-20 14:25:51Z benoitf $
  * --------------------------------------------------------------------------
  */
 
 
 package org.ow2.jonas.dbm.internal.cm;
 
-import java.lang.reflect.Method;
-import java.lang.reflect.Proxy;
-
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.lang.reflect.Proxy;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -42,10 +40,10 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Vector;
-import javax.sql.StatementEventListener;
-import javax.sql.StatementEvent;
+
 import javax.sql.ConnectionEvent;
 import javax.sql.ConnectionEventListener;
+import javax.sql.StatementEventListener;
 import javax.sql.XAConnection;
 import javax.transaction.Synchronization;
 import javax.transaction.Transaction;
@@ -53,11 +51,10 @@ import javax.transaction.xa.XAException;
 import javax.transaction.xa.XAResource;
 import javax.transaction.xa.Xid;
 
-import org.ow2.jonas.lib.util.Log;
-import org.ow2.jonas.tm.Enlistable;
-
 import org.objectweb.util.monolog.api.BasicLevel;
 import org.objectweb.util.monolog.api.Logger;
+import org.ow2.jonas.lib.util.Log;
+import org.ow2.jonas.tm.Enlistable;
 
 /**
  * This class represents the physical connection, managed by the pool.
@@ -147,8 +144,8 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * When that connection was used for an application (with open == 0).
      */
     private long openingTime = DEFAULT_OPENING_TIME;
-    
-    
+
+
 
     /**
      * Used internally for the PrepareStatement cache.
@@ -174,12 +171,12 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * List of Thread infos of clients that released this connection.
      */
     private List closerThreadInfos = new ArrayList();
-    
+
     //  -----------------------------------------------------------------
     //  Constructors
     //  -----------------------------------------------------------------
 
-    public JManagedConnection(Connection conn, ConnectionManager ds) {
+    public JManagedConnection(final Connection conn, final ConnectionManager ds) {
 
         logger.log(BasicLevel.DEBUG, "constructor");
 
@@ -220,7 +217,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * Dynamically change the prepared statement pool size
      */
-    public void setPstmtMax(int max) {
+    public void setPstmtMax(final int max) {
         pstmtmax = max;
         if (psList == null) {
             psList = Collections.synchronizedMap(new HashMap(pstmtmax));
@@ -240,7 +237,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param onePhase true if one phase commit
      * @throws XAException XA protocol error
      */
-    public void commit(Xid xid, boolean onePhase) throws XAException {
+    public void commit(final Xid xid, final boolean onePhase) throws XAException {
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-COMMIT for " + xid);
         }
@@ -296,7 +293,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param flags currently unused
      * @throws XAException XA protocol error
      */
-    public void end(Xid xid, int flags) throws XAException {
+    public void end(final Xid xid, final int flags) throws XAException {
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-END for " + xid);
         }
@@ -308,7 +305,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param xid transaction xid
      * @throws XAException XA protocol error
      */
-    public void forget(Xid xid) throws XAException {
+    public void forget(final Xid xid) throws XAException {
         // not implemented.
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-FORGET for " + xid);
@@ -335,7 +332,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @return True if same RM instance, otherwise false.
      * @throws XAException XA protocol error
      */
-    public boolean isSameRM(XAResource xares) throws XAException {
+    public boolean isSameRM(final XAResource xares) throws XAException {
 
 
         // In this pseudo-driver, we must return true only if
@@ -362,7 +359,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param xid transaction xid
      * @throws XAException XA protocol error
      */
-    public int prepare(Xid xid) throws XAException {
+    public int prepare(final Xid xid) throws XAException {
 
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-PREPARE for " + xid);
@@ -377,7 +374,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @return an array of transaction Xids
      * @throws XAException XA protocol error
      */
-    public Xid[] recover(int flag) throws XAException {
+    public Xid[] recover(final int flag) throws XAException {
 
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-RECOVER for " + this);
@@ -392,7 +389,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param xid transaction xid
      * @throws XAException XA protocol error
      */
-    public void rollback(Xid xid) throws XAException {
+    public void rollback(final Xid xid) throws XAException {
 
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-ROLLBACK for " + xid);
@@ -450,7 +447,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @return always true
      * @throws XAException XA protocol error
      */
-    public boolean setTransactionTimeout(int seconds) throws XAException {
+    public boolean setTransactionTimeout(final int seconds) throws XAException {
 
         if (logger.isLoggable(BasicLevel.DEBUG)) {
             logger.log(BasicLevel.DEBUG, "setTransactionTimeout " + this);
@@ -464,7 +461,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * @param xid transaction xid
      * @throws XAException XA protocol error
      */
-    public void start(Xid xid, int flags) throws XAException {
+    public void start(final Xid xid, final int flags) throws XAException {
 
         if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
             loggerxa.log(BasicLevel.DEBUG, "XA-START for " + xid);
@@ -498,7 +495,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * Compare is based on the number of PrepareStatement cached in this Connection.
      */
-    public int compareTo(Object o) {
+    public int compareTo(final Object o) {
         JManagedConnection other = (JManagedConnection) o;
         int diff = psNumber() - other.psNumber();
         if (diff == 0) {
@@ -548,7 +545,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      *
      * @param listener event listener
      */
-    public void addConnectionEventListener(ConnectionEventListener listener) {
+    public void addConnectionEventListener(final ConnectionEventListener listener) {
         logger.log(BasicLevel.DEBUG, "");
         eventListeners.addElement(listener);
     }
@@ -558,7 +555,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      *
      * @param listener event listener
      */
-    public void removeConnectionEventListener(ConnectionEventListener listener) {
+    public void removeConnectionEventListener(final ConnectionEventListener listener) {
         logger.log(BasicLevel.DEBUG, "");
         eventListeners.removeElement(listener);
     }
@@ -566,11 +563,11 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * Enlist Connection in the Transaction
      */
-    public void enlistConnection(Transaction transaction) throws javax.transaction.SystemException  {
+    public void enlistConnection(final Transaction transaction) throws javax.transaction.SystemException  {
         try {
             if (rme) {
                 if (implConn == null) {
-                    loggerxa.log(BasicLevel.ERROR, "Cannot enlist a closed connection");
+                    loggerxa.log(BasicLevel.WARN, "Cannot enlist a closed connection");
                     return;
                 }
                 // the tx value of the pool item does not match with the enlisted value !?
@@ -595,7 +592,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * delistConnection: used for Stateful beans at postInvoke.
      */
-    public void delistConnection(Transaction transaction) {
+    public void delistConnection(final Transaction transaction) {
         try {
             if (rme) {
                 if (loggerxa.isLoggable(BasicLevel.DEBUG)) {
@@ -605,7 +602,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
                 enlistedInTx = null;
             }
         } catch (Exception e) {
-            loggerxa.log(BasicLevel.ERROR, "Cannot delist Resource:" + e);
+            loggerxa.log(BasicLevel.WARN, "Cannot delist Resource:" + e);
         }
     }
 
@@ -619,7 +616,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      *  synchronization implementation
      */
-    public void afterCompletion(int status) {
+    public void afterCompletion(final int status) {
         if (tx == null ) {
             loggerxa.log(BasicLevel.ERROR, "NO TX!");
         }
@@ -724,7 +721,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
                 // Store the closer's stack
                 closerThreadInfos.add(getThreadInfos());
             }
-        }        
+        }
         if (open < 0) {
             logger.log(BasicLevel.INFO, "connection was already closed");
             open = 0;
@@ -771,12 +768,12 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
 
         return infos;
     }
-    
+
     /**
      * Set the associated transaction
      * @param tx Transaction
      */
-    public void setTx(Transaction tx) {
+    public void setTx(final Transaction tx) {
         this.tx = tx;
     }
 
@@ -797,7 +794,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * set/unset as RME
      */
-    public void setRME(boolean rme) {
+    public void setRME(final boolean rme) {
         this.rme = rme;
     }
 
@@ -806,7 +803,11 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      */
     public void remove() {
         // Make sure that the Connection is logically closed
-        release();
+        // Release only if it is being used
+        if (open > 0) {
+            release();
+        }
+
         // Close the physical connection
         try {
             close();
@@ -826,7 +827,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * Try to find a PreparedStatement in the pool
      */
-    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
+    public PreparedStatement prepareStatement(final String sql, final int resultSetType, final int resultSetConcurrency)
         throws SQLException {
 
         if (actConn == null) {
@@ -835,7 +836,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
             sqle.printStackTrace();
             throw sqle;
         }
-        
+
         loggerps.log(BasicLevel.DEBUG, sql);
         if (pstmtmax == 0) {
             return actConn.prepareStatement(sql, resultSetType, resultSetConcurrency);
@@ -864,7 +865,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /**
      * Try to find a PreparedStatement in the pool
      */
-    public PreparedStatement prepareStatement(String sql) throws SQLException {
+    public PreparedStatement prepareStatement(final String sql) throws SQLException {
         return prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
     }
 
@@ -872,7 +873,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * A PreparedStatement has been logically closed.
      * @return
      */
-    public void notifyPsClose(JStatement ps) {
+    public void notifyPsClose(final JStatement ps) {
         loggerps.log(BasicLevel.DEBUG, ps.getSql());
 
         // Notify event to listeners
@@ -915,7 +916,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /*
      * Notify a Close error statement
      */
-    public synchronized void notifyStatementError(PreparedStatement statement, SQLException e) {
+    public synchronized void notifyStatementError(final PreparedStatement statement, final SQLException e) {
         logger.log(BasicLevel.DEBUG, "");
 
         // Notify event to listeners
@@ -943,7 +944,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
     /*
      * Notify an Error event on Connection
      */
-    public void notifyError(SQLException ex) {
+    public void notifyError(final SQLException ex) {
         logger.log(BasicLevel.DEBUG, "");
 
         // Notify event to listeners
@@ -963,7 +964,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * <p>
      * @since 1.6
      */
-    public void addStatementEventListener(StatementEventListener listener) {
+    public void addStatementEventListener(final StatementEventListener listener) {
         statementEventListeners.add(listener);
     }
 
@@ -976,7 +977,7 @@ public class JManagedConnection implements Comparable, XAConnection, XAResource,
      * <p>
      * @since 1.6
      */
-    public void removeStatementEventListener(StatementEventListener listener) {
+    public void removeStatementEventListener(final StatementEventListener listener) {
         statementEventListeners.remove(listener);
     }
 }
